@@ -6,6 +6,7 @@ from .base import Package
 from .clone import Clone
 from .github import GitHubPackage
 from .links import Links
+from .npm import NpmPackage
 from .pip import PipPackage
 from .pipx import PipxPackage
 from .shell import Shell
@@ -18,6 +19,7 @@ TYPES: list[Tuple[str, Type[Package]]] = [
     ("repo", GitHubPackage),
     ("url", URLPackage),
     ("clone", Clone),
+    ("npm", NpmPackage),
     ("pip", PipPackage),
     ("pipx", PipxPackage),
     ("shell", Shell),
