@@ -1,27 +1,33 @@
-from abc import ABCMeta, abstractmethod
+from abc import ABC, abstractmethod
+from functools import cached_property
 from typing import Optional
 
+from pydantic import BaseModel, Field
+
 from ..driver import Driver
+from ..parallel import gather_
 from ..state import DB
-from ..utils import Some, async_cached, gather, unsome_
+from ..utils import allow_singular, async_cached
 
 
-class Package(metaclass=ABCMeta):
-    os: Optional[list[str]]
-    distribution: Optional[list[str]]
+class Package(BaseModel, ABC):
+    class Config:
+        allow_mutation = False
+        arbitrary_types_allowed = True
+        keep_untouched = (cached_property,)
 
-    def __init__(
-        self,
-        *,
-        db: DB,
-        driver: Driver,
-        os: Some[str] = None,
-        distribution: Some[str] = None,
-    ) -> None:
-        self.db = db
-        self.driver_ = driver
-        self.os = unsome_(os)
-        self.distribution = unsome_(distribution)
+    def __hash__(self) -> int:
+        """Allow hashing to pass when gathering async results."""
+        return hash(id(self))
+
+    os: Optional[list[str]] = None
+    os_val = allow_singular("os")
+
+    distribution: Optional[list[str]] = None
+    distribution_val = allow_singular("distribution")
+
+    db: DB
+    driver_: Driver = Field(..., alias="driver")
 
     @property
     def driver(self) -> Driver:
@@ -45,7 +51,7 @@ class Package(metaclass=ABCMeta):
         pass
 
     async def is_installed(self) -> bool:
-        remote, local = await gather(self.remote_version, self.local_version)
+        remote, local = await gather_(self.remote_version, self.local_version)
         return remote == local
 
     @abstractmethod
