@@ -159,14 +159,16 @@ class DNF(PackageCacheInstaller):
             versions[name] = version
 
         if package is not None:
+            if package in versions:
+                return {package: versions[package]}
             # If querying for a specific package, the name might be different
             # because of virtual packages and --whatprovides option.
-            # Assume the only version and extract it.
+            # If there's no package with exact name, require a single match.
             if len(versions) > 1:
                 raise ValueError(f"Multiple versions for {package}: {versions}.")
             if len(versions) == 0:
                 raise ValueError(f"No versions for {package}.")
-            (version,) = versions.values()
+            (version,) = versions.values()  # pylint:disable=unbalanced-dict-unpacking
             return {package: version}
 
         return versions
@@ -239,7 +241,9 @@ class DNF(PackageCacheInstaller):
 
 class Apt(Installer):
     async def install(self, package: str) -> None:
-        await self.driver.with_root(True).run("apt", "install", "--yes", package)
+        await self.driver.with_root(True).run(
+            "env", "DEBIAN_FRONTEND=noninteractive", "apt", "install", "--yes", package
+        )
 
     async def upgrade(self, package: str) -> None:
         await self.install(package)
