@@ -1,21 +1,12 @@
-from abc import ABCMeta
-from dataclasses import dataclass
+from abc import ABC
 from functools import cached_property
 from typing import Optional
 
-from ..state import Storage, storage
+from ..state import VERSIONS, Storage, Version
 from .base import Package
 
 
-@dataclass
-class Version:
-    version: str
-
-
-VERSIONS = storage("version", Version)
-
-
-class ManualVersion(Package, metaclass=ABCMeta):
+class ManualVersion(Package, ABC):
     @cached_property
     def versions(self) -> Storage[Version]:
         return VERSIONS(self.db)
