@@ -8,9 +8,7 @@ SHELLS_FILE = Path("/etc/shells")
 
 
 class Shell(Root):
-    def __init__(self, shell: str, **kwargs) -> None:
-        super().__init__(**kwargs)
-        self.shell = Path(shell)
+    shell: Path
 
     @property
     def name(self) -> str:
@@ -54,6 +52,6 @@ class Shell(Root):
                 "tee", "-a", SHELLS_FILE, input=str(self.shell).encode()
             )
 
-        await self.driver.run("chsh", "-s", self.shell)
+        await self.driver.run("chsh", "-s", self.shell, show_output=True)
 
         await super().install()
