@@ -1,8 +1,10 @@
 from typing import Optional
 
 import trio
+from pydantic import Field
 
-from ..utils import Some, unsome, url_version
+from ..compute import Value
+from ..utils import allow_singular_none, async_cached, url_version
 from .installer import make_installer
 from .manual_version import ManualVersion
 
@@ -10,37 +12,34 @@ INSTALLER_LOCK = trio.Lock()
 
 
 class SystemPackage(ManualVersion):
-    def __init__(
-        self,
-        *,
-        name: str,
-        url: Optional[str] = None,
-        auto_updates: bool = False,
-        service: Some[str] = None,
-        **kwargs,
-    ) -> None:
-        super().__init__(**kwargs)
-        self._name = name
-        self.url = url
-        self.auto_updates = auto_updates
-        self.services = unsome(service)
+    name_: str = Field(..., alias="name")
+    url_: Optional[Value] = Field(default=None, alias="url")
+    auto_updates: bool = False
+    services: list[str] = Field(default_factory=list, alias="service")
+    services_val = allow_singular_none("services")
 
     async def installer(self):
         return await make_installer(self.driver)
 
     @property
     def name(self) -> str:
-        return self._name
+        return self.name_
+
+    @async_cached
+    async def url(self) -> Optional[str]:
+        if self.url_:
+            return await self.url_.compute()
+        return None
 
     async def get_remote_version(self) -> str:
-        if self.url:
-            return url_version(self.url)
+        if url := await self.url():
+            return url_version(url)
         if self.auto_updates:
             return "latest"
         return await (await self.installer()).latest_version(self.name)
 
     async def local_version(self) -> Optional[str]:
-        if self.url:
+        if await self.url():
             return self.cached_version
         installer = await self.installer()
         version = await installer.installed_version(self.name)
@@ -60,8 +59,8 @@ class SystemPackage(ManualVersion):
     async def install(self) -> None:
         async with INSTALLER_LOCK:
             installer = await self.installer()
-            if self.url:
-                await installer.install(self.url)
+            if url := await self.url():
+                await installer.install(url)
                 await self.cache_version()
             elif await self.local_version():
                 await installer.upgrade(self.name)
