@@ -3,9 +3,7 @@ from .archive import ArchivePackage
 
 
 class URLPackage(ArchivePackage):
-    def __init__(self, *, url: str, **kwargs) -> None:
-        self.url = url
-        super().__init__(**kwargs)
+    url: str
 
     async def archive_url(self) -> str:
         return self.url
@@ -16,12 +14,10 @@ class URLPackage(ArchivePackage):
         while True:
             if len(parts) == 0:
                 raise ValueError(f"Cannot parse package name from {self.url}.")
-            elif parts[0] in ("", "https:"):
+            if parts[0] in ("", "https:", "github.com"):
                 parts.pop(0)
-            elif parts[0] == "github.com":
-                return "/".join(parts[1:2])
-            else:
-                return parts[0]
+                continue
+            return "/".join(parts[1:2])
 
     async def get_remote_version(self) -> str:
         return url_version(self.url)
