@@ -3,10 +3,25 @@ import sqlite3
 import threading
 from abc import ABCMeta, abstractmethod
 from pathlib import Path
-from typing import Any, Callable, Generic, Iterable, Tuple, Type, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Generic,
+    Iterable,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 from uuid import uuid4
 
-T = TypeVar("T")
+if TYPE_CHECKING:
+    from _typeshed import DataclassInstance  # pragma: no cover
+
+    M = TypeVar("M", bound=DataclassInstance)  # pragma: no cover
+else:
+    M = TypeVar("M")
 
 DB_PATH = "state.sqlite"
 
@@ -31,9 +46,9 @@ class DB:
         return connection
 
 
-class Storage(Generic[T], metaclass=ABCMeta):
+class Storage(Generic[M], metaclass=ABCMeta):
     @abstractmethod
-    def __getitem__(self, key: str) -> T:
+    def __getitem__(self, key: str) -> M:
         pass
 
     @abstractmethod
@@ -41,34 +56,38 @@ class Storage(Generic[T], metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def __setitem__(self, key: str, value: T) -> None:
+    def __setitem__(self, key: str, value: M) -> None:
         pass
 
     @abstractmethod
-    def append(self, value: T) -> None:
+    def append(self, value: M) -> None:
         pass
 
     @abstractmethod
-    def find(self, **kwargs: Any) -> Iterable[T]:
+    def find_ids(self, **kwargs: Any) -> Iterable[tuple[str, M]]:
         pass
 
+    def find(self, **kwargs: Any) -> Iterable[M]:
+        for _, value in self.find_ids(**kwargs):
+            yield value
+
     @abstractmethod
     def delete(self, **kwargs: Any) -> None:
         pass
 
 
-StorageDefinition = Callable[[DB], Storage[T]]
+StorageDefinition = Callable[[DB], Storage[M]]
 
 
-def storage(name: str, klass: Type[T]) -> StorageDefinition[T]:
+def storage(name: str, klass: Type[M]) -> StorageDefinition[M]:
     attributes: list[str] = [field.name for field in dataclasses.fields(klass)]
 
-    def storage_table(db: DB) -> Storage[T]:
+    def storage_table(db: DB) -> Storage[M]:
         db.instance.execute(
             f'CREATE TABLE IF NOT EXISTS {name} (id TEXT PRIMARY KEY, {", ".join(attributes)})'
         )
 
-        class StorageImpl(Storage[T]):
+        class StorageImpl(Storage[M]):
             def where_clause(self, **kwargs: Any) -> Tuple[str, list[Any]]:
                 result = "1=1"
                 values = []
@@ -77,7 +96,7 @@ def storage(name: str, klass: Type[T]) -> StorageDefinition[T]:
                     values.append(value)
                 return result, values
 
-            def find(self, **kwargs: Any) -> Iterable[T]:
+            def find_ids(self, **kwargs: Any) -> Iterable[tuple[str, M]]:
                 where, values = self.where_clause(**kwargs)
 
                 cursor = db.instance.execute(
@@ -89,9 +108,9 @@ def storage(name: str, klass: Type[T]) -> StorageDefinition[T]:
                         attributes = {
                             key: row[key] for key in row.keys() if key != "id"
                         }
-                        yield klass(**attributes)
+                        yield row["id"], klass(**attributes)
 
-            def __getitem__(self, key: str) -> T:
+            def __getitem__(self, key: str) -> M:
                 items = self.find(id=key)
                 try:
                     return next(iter(items))
@@ -105,7 +124,7 @@ def storage(name: str, klass: Type[T]) -> StorageDefinition[T]:
                 where, values = self.where_clause(**kwargs)
                 db.instance.execute(f"DELETE FROM {name} WHERE {where}", values)
 
-            def __setitem__(self, key: str, value: T) -> None:
+            def __setitem__(self, key: str, value: M) -> None:
                 with db.instance:
                     del self[key]
                     attr_clause = "?"
@@ -117,7 +136,7 @@ def storage(name: str, klass: Type[T]) -> StorageDefinition[T]:
                         f"INSERT INTO {name} VALUES ({attr_clause})", attr_values
                     )
 
-            def append(self, value: T) -> None:
+            def append(self, value: M) -> None:
                 self[str(uuid4())] = value
 
         return StorageImpl()
