@@ -1,14 +1,11 @@
-import re
 import subprocess
 from functools import wraps
 from pathlib import Path
 from typing import (
     Any,
-    Awaitable,
     Callable,
     Coroutine,
     Iterable,
-    Iterator,
     Optional,
     TypeVar,
     Union,
@@ -16,8 +13,8 @@ from typing import (
 )
 
 import requests
-import tqdm  # type: ignore
 import trio
+from pydantic import validator
 
 T = TypeVar("T")
 U = TypeVar("U")
@@ -39,6 +36,14 @@ def unsome(x: Some[T]) -> list[T]:
     return unsome_(x) or []
 
 
+def allow_singular(field: str) -> Any:
+    return validator(field, pre=True, allow_reuse=True)(lambda cls, x: unsome_(x))
+
+
+def allow_singular_none(field: str) -> Any:
+    return validator(field, pre=True, allow_reuse=True)(lambda cls, x: unsome(x))
+
+
 def intercalate(delimiter: T, items: Iterable[Iterable[T]]) -> list[T]:
     it = iter(items)
     try:
@@ -51,9 +56,6 @@ def intercalate(delimiter: T, items: Iterable[Iterable[T]]) -> list[T]:
     return result
 
 
-TERMINAL_LOCK = trio.Lock()
-
-
 RunArg = Union[str, Path]
 
 
@@ -82,82 +84,6 @@ def flatten(items: Iterable[Iterable[T]]) -> list[T]:
     return [item for sublist in items for item in sublist]
 
 
-async def parallel_map_tqdm(
-    action: Callable[[T], Awaitable[U]], items: list[T]
-) -> list[U]:
-    results: list[U] = []
-
-    with tqdm.tqdm(total=len(items)) as progress:
-        async with trio.open_nursery() as nursery:
-
-            async def action_and_update(item: T) -> None:
-                result = await action(item)
-                async with TERMINAL_LOCK:
-                    progress.update(1)
-                results.append(result)
-
-            for item in items:
-                nursery.start_soon(action_and_update, item)
-
-            return results
-
-
-async def gather(*tasks: Callable[[], Awaitable[T]]) -> list[T]:
-    async def collect(
-        index: int, task: Callable[[], Awaitable[T]], results: dict[int, T]
-    ):
-        results[index] = await task()
-
-    results: dict[int, T] = {}
-    async with trio.open_nursery() as nursery:
-        for index, task in enumerate(tasks):
-            nursery.start_soon(collect, index, task, results)
-    return [results[i] for i in range(len(tasks))]
-
-
-class Filters:
-    @staticmethod
-    def includes(substring: str) -> Callable[[str], bool]:
-        return lambda x: substring in x.lower()
-
-    @staticmethod
-    def excludes(substring: str) -> Callable[[str], bool]:
-        return lambda x: substring not in x
-
-    @staticmethod
-    def startswith(prefix: str) -> Callable[[str], bool]:
-        return lambda x: x.startswith(prefix)
-
-    @staticmethod
-    def endswith(suffix: str) -> Callable[[str], bool]:
-        return lambda x: x.endswith(suffix)
-
-    @staticmethod
-    def regex(regex: str) -> Callable[[str], bool]:
-        regex_compiled = re.compile(regex)
-
-        return lambda x: regex_compiled.match(x) is not None
-
-
-def choose(candidates: list[T], filters: Iterator[Callable[[T], bool]]) -> T:
-    if len(candidates) == 0:
-        raise ValueError("No candidates to choose from.")
-    while len(candidates) > 1:
-        try:
-            filter_fn = next(filters)
-        except StopIteration:
-            break
-
-        new_candidates = list(filter(filter_fn, candidates))
-        if new_candidates:
-            candidates = new_candidates
-
-    if len(candidates) == 1:
-        return candidates[0]
-
-    raise ValueError(f"Cannot choose between: {candidates}.")
-
-
 def url_version(url: str) -> str:
     head_response = requests.head(url, allow_redirects=True)
     return head_response.headers["etag"]
